function [f_in_x0] = Fct_f_in_xpoint_jointanglestimes_80211az(x0,AP_positions, c, start_time, forward_time)
% Build the observables in x0 point, when joint time of flight and angle
% observations are used; f_in_x0 will be a column vector 3*N_AP x 1, where
% first N_AP measurements correspond to the time, the next N_AP correspond
% to elevation angle, and the last N_AP correspond to azimuth anngle
%AP_positions = N_AP x 3 matrix with each row showing the x,y,z coordinates
%               of the ap-th access point
%c             = speed of light at the level used in the main file, e.g. in
%                m/ns
N_AP=size(AP_positions, 1);
dif_vect=zeros(N_AP,3);
dif_ap_vect=zeros(N_AP,3);
f_in_x0=zeros(1,3*N_AP);

%TIME Observables
% Calculating the observables
sum_of_distancesAP=0;
for i=1:N_AP
    dif_vect(i,:)=x0-AP_positions(i,:); %distance between AP i and MS
    if i>1,
        dif_ap_vect(i,:)=AP_positions(i,:)-AP_positions(i-1,:);
    else
        dif_ap_vect(i,:)=zeros(1,3);
    end;
    %distance between AP index i and AP index i-1;
    distAPi_APiplus1=sqrt(dif_ap_vect(i,1)^2+dif_ap_vect(i,2)^2+dif_ap_vect(i,3)^2);
    sum_of_distancesAP=sum_of_distancesAP+distAPi_APiplus1;
    %time = start time + forwarding time*(i-1) + sum_of_distancesAP/c +
    %distance between AP i and MS/c
    f_in_x0(i) = start_time +  (i-1)*forward_time+ sum_of_distancesAP/c+ (sqrt(dif_vect(i,1)^2 + dif_vect(i,2)^2 + dif_vect(i,3)^2))/c  ;
end;
f_in_x0=f_in_x0/1000;

%ELEVATION angle observables or theta
for i=N_AP+1:2*N_AP,
     dif_vect(i,:)=x0-AP_positions(i-N_AP,:);
     %distance between AP index i and the x0 points
     dist_APi_MS=sqrt(dif_vect(i,1)^2+dif_vect(i,2)^2+dif_vect(i,3)^2);
     f_in_x0(i) =    asin(abs((x0(3)-AP_positions(i-N_AP,3)))/dist_APi_MS);  
end;

%AZIMUTH angle observables or theta
for i=2*N_AP+1:3*N_AP,
     f_in_x0(i) =atan2(abs((x0(1)-AP_positions(i-2*N_AP,1))),abs((x0(2)-AP_positions(i-2*N_AP,2)))); % Atan de X/Y
end;