function [mvl,xest1,xest2,xest3,dist_err_time_estim,dist_err_angle_estim,dist_err_joint_estim]=mobile_postion_estimation_joint_time_angles(N_AP,var_t,var_az,var_elev,mu_t,mu_az,mu_elev,v)

%% Drawing a building and place randomly the Access Points
% Cleaning workspace
warning off
%% Definition of the variables

% Firstly, indicate the number of AP's you want to consider in the study
% AP_study=input('Indicate the number of APs you want to consider in the study ');  % PODRAMOS CAMBIAR ESTO PARA QUE SE PUDIERA INTRODUCIR UN VECTOR CON EL NMERO DE AP'S QUE SE QUIEREN ESTUDIAR
% N_iter=input('Indicate the number of iterations you want to make in each realization ');

% The minimun AP can be considered is 3. Starting from that simple
% structure:
% AP_min=3;
% Num_AP=[AP_min:AP_study]; % Array containing the Numbers of APs to consider in each realization of the study
% 
% Initialize the seed of the random generator to place in same position every time we run the code
% rng(0,'twister');

% Time Forwarding
delayF=10; % ns
% Time of start
ts=0;

% Noise Parameters
% mu=0; % In the begining this value is 0 to get a LOS environment. If mu != 0 -> NLOS environment
% var_t=0.1; % The variance's value for time observation is ct and equal for all the APs; is it given in ns
% var_a=0.01; % The variance's value for angle observation is ct and equal for all the APs, it is given in rad

% Speed of light in m/nanoseconds
c=physconst('LightSpeed')/1e9; % m/ns

% Error Vectors
dist_err_vector_joint_estim=[];
dist_err_vector_time_estim=[];
dist_err_AP_joint=[];
dist_err_AP_time=[];


%% Evaluating the influence of the number of Access Points and the number of iterations the code has run
        %% 1st: Drawing the building, Place randomly the AP's and the mobile
        % Building
        x0=1;
        y0=1;
        z0=0;

        Lx=100; % X width
        Ly=100; % Y width
        Lz=5; % 5m of height
        Nfloors=3;
        for nf=1:Nfloors,
            box_3D_interface(x0, y0, z0+(nf-1)*Lz, Lx, Ly, Lz,v); % floor box
        end;

        % Place randomly the AP's in the building
        xy_vector=randi(100,N_AP,2); % Generate 3 vectors of 2 elements for xy coordinates of the Access Points
        AP_positions=zeros(N_AP, 3); %vector of AP positions
        for nap=1:N_AP,
            AP_positions(nap,:)=[xy_vector(nap,1), xy_vector(nap,2), (randi(Nfloors, 1,1)-1)*Lz]; 
            hold on;
            axes(v)
            plot3(AP_positions(nap,1), AP_positions(nap,2), AP_positions(nap,3),'r','MarkerSize',15,'Marker','diamond');
        end;

        % Place randomly the mobile in the building
        xy_mob_vector=randi(100,1,2);
        floor_mob=randi(Nfloors); % Indicates the floor where the mobile is going to be placed
        mvl=[xy_mob_vector(1), xy_mob_vector(2), (floor_mob-1)*Lz];
        
        axes(v)
        plot3(mvl(1), mvl(2), mvl(3), 'g','MarkerSize',15,'Marker','square','LineWidth',3.0);

        %% 2nd: Frames are sent to the network -> Mobile can now use it to compute time and estimate position
        % The frames contain the following parameters:
        % - ts: time of start (in this case it is assumed is 0 ns)
        % - tf: a delay relative to the time of start
        % - AP's positions: x,y,z coordinates of the AP's

        %% 3rd: Creating observables t1, t2, ... tn
        % knowing the position of the mobile and the access points, we create the
        % observables adding a random noise to randomize the process

        % Equation used: t1 = ts + d(M,AP1)/c + n
        %                t2 = ts + d(M,AP2)/c + d(AP2, AP1)/c + tf + n

        % The noise added in the above equation has the following characteristics:
        % - zero means and constant variance (and equal for all observables)

        % Creating the vectors will be used
        t=[zeros(1,N_AP)]; % Vector which contains the time observables
        dif_vect=[zeros(N_AP,3)]; % Vector which contains the difference between mobile position to each AP position
        dif_ap_vect=[zeros(N_AP,3)]; % Vector which contains the difference between the position of consecutive APs
        distances_APS=zeros(1,N_AP); % Vector which contains the distance between each consecutive AP. Using the matrix above.
        sum_of_distancesAP=0; % Vector which contains the acumulative distance between AP's. It will be the sum of d(APi,APi-1)+...
        obs_thetaangles=zeros(1,N_AP); % Vector which contains the theta angles observables (elevation angle)
        obs_alphaangles=zeros(1,N_AP); % Vector which contains the alpha angles observables (azimuth angle)

        % Calculating the observables
        for i=1:N_AP
            dif_vect(i,:)=mvl-AP_positions(i,:); % Difference between position of AP i and MS
            dist_APi_MS=sqrt(dif_vect(i,1)^2+dif_vect(i,2)^2+dif_vect(i,3)^2);
            if i>1,
                dif_ap_vect(i,:)=AP_positions(i,:)-AP_positions(i-1,:); % Difference between APi and APi-1 position
            else
                dif_ap_vect(i,:)=zeros(1,3); % If the observable calcaluted is the number 1, there is no distance between him and a consecutive AP to be considered
            end;

            % Distance between AP index i and AP index i-1;
            distAPi_APiplus1=sqrt(dif_ap_vect(i,1)^2+dif_ap_vect(i,2)^2+dif_ap_vect(i,3)^2); % Using the matrix where is saved the difference between positions of consecutive AP's
            sum_of_distancesAP=sum_of_distancesAP+distAPi_APiplus1; % Acumulative distance d(AP1,AP2)+d(AP2,AP3)...

            % time = start time + forwarding time*(i-1) + sum_of_distancesAP/c + distance between AP i and MS/c + noise
            t(i) = ts +  (i-1)*delayF+ sum_of_distancesAP/c+ (sqrt(dif_vect(i,1)^2 + dif_vect(i,2)^2 + dif_vect(i,3)^2))/c  + (mu_t+sqrt(var_t)*randn(1,1)) ;
            
            % Theta angle = arcsin((AP_z(i)-z0)/distance between AP i and MS)
            obs_thetaangles(i)=asin(abs((mvl(3)-AP_positions(i,3)))/dist_APi_MS) + (mu_elev+sqrt(var_elev)*randn(1,1)); % Elevation angle

            % Alpha angle = arctan(AP_y(i)-y0/AP_x(i)-x0)
            obs_alphaangles(i)=atan2(abs((mvl(1)-AP_positions(i,1))),abs((mvl(2)-AP_positions(i,2)))) + (mu_az+sqrt(var_az)*randn(1,1)); % Azimuth angle

        end
        
        %% 4th: Estimation of the position of the mobile

            %% Newton-Raphson method
        % Applying Newton-Raphson method we will obtain the position of the mobile,
        % evaluating the observable function in different points until the method
        % converges.

        % Functions to implement:
        % - Observable functions (t1, t2, .. t_NAP)
        % - Jacobian (obtained applying the diff to the function of the observables)

        %to check how to deal with Mx N size Jacobian with M~=N
        % % INITIAL POINT: 
%         xold=[mean(AP_positions)']; % Starting point based on the mean of the AP's positions
%         Niter=10;
%         dist_err=NaN*ones(1,Niter); %error in the estimation
%         for kiter=1:Niter,
%             [Jacobian] = Fct_buildJacobian_80211az(xold,AP_positions, c);
%             [f_in_x0] = Fct_build_f_inxnpoint_80211az(xold',AP_positions, c, ts, delayF);
%             xnew=xold+inv(Jacobian)*(t-f_in_x0)'; % inv(jacobian)*(t-f_in_x0)' has plus sign because we are considering the difference: t-f_in_x0 not f_in_x0-t (t_est-t)
%             xold=xnew;
%             hold on; plot3(xnew(1), xnew(2), xnew(3), 'r','MarkerSize',10,'Marker','x','LineWidth',2.0); drawnow;
%             dist_err(kiter)=sqrt((xnew(1)-mvl(1)).^2+(xnew(2)-mvl(2)).^2+(xnew(3)-mvl(3)).^2);
%         end;
% 

            %% Non linear squares method
        % Due to the fact N-R doesn't converge, applying NLS method we will obtain
        % the position of the mobile.
        % Using the function "lsqcurvefit" available in matlab, we will obtain the
        % postion of the mobil which solve the following equation: min(abs(F(x,xdata)-ydata)^2)

        % Where:
            % x - are the x,y,z coordinates of the mobile to find
            % xdata - are the x,y,z coordinates of the each AP
            % ydata - are the observables

        % This function needs some input parameters:
        % - fun: function we want to fit. In that case, it will be the  estimated observables function (joint time and angle estimation or just time estimation)
        % - x0: initial point
        % - xdata: input data for the model. In our case, they will be the AP positions
        % - ydata: response data for model. It is important that ydata has the same size and shape that the fun(x0,xdata). In our case, they will be the created observables.
        % - options: optimization options. It can be changed the algorithm used in the minimization process, the tolerance to converge and more parameters

        % Defining Input parameters
        fun1=@(x,xdata)( Fct_build_f_inxnpoint_80211az(x',xdata, c, ts, delayF)); % only the time estimation
        fun2=@(x,xdata)( Fct_f_in_xpoint_angles_80211az(x',xdata)); % only angle estimation
        fun3=@(x,xdata)( Fct_f_in_xpoint_jointanglestimes_80211az(x',xdata, c, ts, delayF)); % to create the function of the joint time and angle estimation
        
        % initial guess is taken as the centroid of AN
        x0=[mean(AP_positions)']; 
        
        % Response data for model fun2: angle observables
        aobs=[obs_thetaangles obs_alphaangles];
        
        % Response data for model fun3: joint time and angle observables
        yobs=[t/1000 obs_thetaangles obs_alphaangles];

        % The algorithm will be selected is LEVENBERG-MARQUARDT, due to the fact that in
        % the cases that N-R does not converge, Levenberg-Marquardt algorithm is
        % more suitable
        options=optimoptions(@lsqcurvefit,'Algorithm','levenberg-marquardt','MaxIter',1000); % Default value 'TolX',1e-6 'TolFun',1e-10,'TolX',1e-12

        % Calling to the NLS method
%         try
%             [xest3]=lsqcurvefit(fun1, x0, AP_positions,yobs,-Inf,+Inf,options); 
%         catch
%             xest3=NaN*ones(1,3);
%         end
%         try
%             [xest1]=lsqcurvefit(fun2, x0, AP_positions,t,-Inf,+Inf,options); 
%         catch 
%             xest1=NaN*ones(1,3);
%         end;
%         try
%             [xest2]=lsqcurvefit(fun2, x0, AP_positions,aobs,-Inf,+Inf,options); 
%         catch 
%             xest2=NaN*ones(1,3);
%         end;
%  
        % Creating the optimization prolems
        problem1 = createOptimProblem('lsqcurvefit','x0',x0,'objective',fun1,...
             'lb',[0 0 0],'ub',[Lx Ly Nfloors*Lz],'xdata',AP_positions,'ydata',t,'options',options);       

        problem2 = createOptimProblem('lsqcurvefit','x0',x0,'objective',fun2,...
             'lb',[0 0 0],'ub',[Lx Ly Nfloors*Lz],'xdata',AP_positions,'ydata',aobs,'options',options);       
                 
        problem3 = createOptimProblem('lsqcurvefit','x0',x0,'objective',fun3,...
             'lb',[0 0 0],'ub',[Lx Ly Nfloors*Lz],'xdata',AP_positions,'ydata',yobs,'options',options);

        % MULTISTART CLASS -> Get the converge of the lsqcurvefit generating
        % different starting points and running the local solver (LMA) to each one.
        ms = MultiStart;

        % Running the multistart method with 50 starting points for each
        % optimization problem
    try    
        [xest1] = run(ms,problem1,50);
        [xest2] = run(ms,problem2,50);
        [xest3] = run(ms,problem3,50);
    catch
        xest1=NaN*ones(1,3);
        xest2=NaN*ones(1,3);
        xest3=NaN*ones(1,3);
    end

        % Calculating the distance error between the estimated position and the
        % real position of the mobile
        dist_err_time_estim=sqrt((xest1(1)-mvl(1)).^2+(xest1(2)-mvl(2)).^2+(xest1(3)-mvl(3)).^2); % Vector with the dist error in each iteration 
        dist_err_angle_estim=sqrt((xest2(1)-mvl(1)).^2+(xest2(2)-mvl(2)).^2+(xest2(3)-mvl(3)).^2);
        dist_err_joint_estim=sqrt((xest3(1)-mvl(1)).^2+(xest3(2)-mvl(2)).^2+(xest3(3)-mvl(3)).^2);

end
